from slugify import slugify

class HomePage:
    def __init__(self, label):
        self.label = label
        self.uri = self.__process_label_for_uri(label)
        self.body = self.__build()

    @staticmethod
    def __process_label_for_uri(title):
        return f"https://abolition-now.github.io/an/works/{slugify(title)}"

    def __build(self):
        return {
            "id": self.uri,
            "type": "Text",
            "label": { "en": [ self.label ] },
            "format": "text/html",
            "language": [ "en" ]
        }


from csv import DictReader
from iiif_now.datacanvas import DataCanvas

class DataReader:
    def __init__(self, filename, artists_file, metadata_file):
        self.filename = filename
        self.artists = self.__dereference_artists(artists_file)
        self.metadata = self.__build_metadata(metadata_file)
        self.relevant_rows = self.__read(filename)

    def __repr__(self):
        return f"DataReader({self.filename})"

    def __str__(self):
        return f"DataReader for {self.filename}"

    @staticmethod
    def __dereference_artists(artists_sheet):
        with open(artists_sheet) as f:
            reader = DictReader(f)
            artists = {row['Artist Code']: row['Artist Name'] for row in reader}
        return artists

    @staticmethod
    def __build_metadata(metadata_sheet):
        with open(metadata_sheet) as f:
            reader = DictReader(f)
            metadata = {}
            for row in reader:
                for k, v in row.items():
                    if v != '':
                        metadata[v] = k
        return metadata

    def build_hierarchy(self):
        """
        Build a hierarchy of canvases based on the parent field in the CSV

        Returns:
            list: A list of dictionaries of canvases organized by parent

        Example:
            [
              {
                'id': '0001_newbeginnings',
                'canvases': [
                   {'label': 'Original Image', 'sequence': '1', 'parent': '0001_newbeginnings', 'type': 'Image', 'thumbnail': '0001_newbeginnings_oo.jpg', 'metadata': {}, 'artists': ['Ricardo Levins Morales']},
                   {'label': 'Reuse', 'sequence': '2', 'parent': '0001_newbeginnings', 'type': 'Image', 'thumbnail': '0001_newbeginnings_ru.png', 'metadata': {'Visual Motif': ['Butterflies']}, 'artists': ['Ricardo Levins Morales']}
                ],
                'manifest_title': 'New Beginnings - Monarch Butterfly',
                'artists': ['Ricardo Levins Morales'],
                'metadata': {'Visual Motif': ['Butterflies']}
              }
            ]
        """
        hierarchy = []
        for row in self.relevant_rows:
            canvas_id = row['parent']
            canvas_dict = next((item for item in hierarchy if item['id'] == canvas_id), None)
            if canvas_dict is None:
                canvas_dict = {
                    'id': canvas_id,
                    'canvases': [],
                    'manifest_title': '',
                    'artists': [],
                    'metadata': {}
                }
                hierarchy.append(canvas_dict)

            canvas = DataCanvas(row, self.artists, self.metadata)
            if canvas.parent_title != '':
                canvas_dict['manifest_title'] = canvas.parent_title
            canvas_dict['canvases'].append(canvas.as_dict)
            # @Todo: Does this need to be here with below?
            for artist in canvas.artists:
                if artist not in canvas_dict['artists']:
                    canvas_dict['artists'].append(artist)
            # @Todo: Break this into separate method
            if canvas.metadata:
                for k, v in canvas.metadata.items():
                    if k not in canvas_dict['metadata']:
                        canvas_dict['metadata'][k] = v
                    else:
                        for value in v:
                            if value not in canvas_dict['metadata'][k]:
                                canvas_dict['metadata'][k].append(value)
            # @Todo: Break this into separate method
            # Add artists to metadata if not already present
            if len(canvas_dict['artists']) > 0:
                if 'Artist' not in canvas_dict['metadata']:
                    canvas_dict['metadata']['Artist'] = canvas_dict['artists']
                else:
                    for artist in canvas_dict['artists']:
                        if artist not in canvas_dict['metadata']['Artist']:
                            canvas_dict['metadata']['Artist'].append(artist)
        for canvas_dict in hierarchy:
            canvas_dict['canvases'] = sorted(canvas_dict['canvases'], key=lambda x: int(x['sequence']))
        return hierarchy


    @staticmethod
    def __read(csv_file):
        relevant_rows = []
        with open(csv_file) as f:
            reader = DictReader(f)
            for row in reader:
                if row.get('parent') != "":
                    relevant_rows.append(row)
        return relevant_rows



from .csvreader import DataReader
from .datacanvas import DataCanvas
from .homepage import HomePage
from .manifest import ANManifest
from .navplace import NavPlace
from .study_guide import StudyGuide
from .thumbnail import Thumbnail


#!/usr/bin/env python3

from iiif_now.csvreader import DataReader
from iiif_now.manifest import ANManifest
import click
import yaml
from tqdm import tqdm

@click.group()
def cli() -> None:
    pass

@cli.command("use", help="Generate IIIF manifests from a AN! data files.")
@click.argument("config", required=True)
def use(config: str) -> None:
    settings = yaml.safe_load(open(config, "r"))
    reader = DataReader(
        settings['main_sheet'],
        artists_file=settings['artists_file'],
        metadata_file=settings['metadata_codes']
    )
    manifests = reader.build_hierarchy()
    for manifest in tqdm(manifests):
        x = ANManifest(
            manifest,
            image_server_path=settings['image_server_path'],
            video_location=settings['video_location'],
            manifest_bucket=settings['manifest_bucket'],
            extensions=settings['extensions']
        )
        x.write(
            settings['manifest_directory']
        )


from csv import DictReader


class DataCanvas:
    def __init__(self, canvas_data, artists_data, metadata_data):
        self.canvas_data = canvas_data
        self.artists_data = artists_data
        self.metadata_data = metadata_data
        self.artists = self.__find_canvas_artists()
        self.metadata = self.__build_metadata()
        self.label = self.canvas_data['canvas title']
        self.sequence = self.get_sequence(self.canvas_data['sequence'])
        self.parent = self.canvas_data['parent']
        self.type = self.canvas_data['type']
        self.thumbnail = self.__find_thumbnail_source()
        self.parent_title = self.canvas_data['title']
        self.as_dict = self.__build_dict()

    @staticmethod
    def get_sequence(value):
        try:
            return int(value)
        except ValueError:
            return 1

    def __find_canvas_artists(self):
        artists = []
        split_key = self.canvas_data['key'].split('_')
        for value in split_key:
            if value in self.artists_data:
                artists.append(self.artists_data[value])
        return artists

    def __build_metadata(self):
        metadata = {}
        for k, v in self.canvas_data.items():
            if k.startswith('code_') and v and v in self.metadata_data:
                field = self.metadata_data[v]
                metadata.setdefault(field, []).append(v)
        return metadata

    def __find_thumbnail_source(self):
        if self.canvas_data['thumbnail source'] != '':
            return self.canvas_data['thumbnail source']
        else:
            return self.canvas_data['key']

    def __build_dict(self):
        return {
            'label': self.label,
            'sequence': self.sequence,
            'parent': self.parent,
            'type': self.type,
            'thumbnail': self.thumbnail,
            'metadata': self.metadata,
            'artists': self.artists,
            'key': self.canvas_data['key'],
            'duration': self.canvas_data['duration']
        }



class StudyGuide:
    def __init__(self, artists):
        self.guides = self.__known_guides()
        self.artists = artists
        self.body = self.__build()

    def __known_guides(self):
        return {
            "Josh MacPhee": {
                "url": "https://drive.google.com/file/d/1aWo1lORRVTQ0VveV3aP5Ym6hfVXUqr8_/view?usp=sharing",
                "label": 'Download "A study guide: Josh MacPhee"'
            }

        }

    def __build(self):
        for artist in self.artists:
            if artist in self.guides:
                return {
                    "id": self.guides[artist]["url"],
                    "type": "Text",
                    "label": { "en": [ self.guides[artist]["label"] ] },
                    "format": "application/pdf"
                }
        return None


from iiif_prezi3 import Manifest, config, KeyValueString, load_bundled_extensions, AnnotationPage, Annotation, ResourceItem
import requests
import json
from iiif_now.homepage import HomePage
from iiif_now.navplace import NavPlace
from iiif_now.study_guide import StudyGuide
from iiif_now.thumbnail import Thumbnail


class ANManifest:
    def __init__(
            self,
            manifest_data,
            image_server_path="https://strob6zro3bzklrulaqu2545sy0odbvz.lambda-url.us-east-2.on.aws/iiif/3/",
            video_location="https://digital.lib.utk.edu/static/",
            manifest_bucket="https://raw.githubusercontent.com/markpbaggett/static_iiif/main/manifests/abolition_now/",
            extensions=[]
    ):
        self.config = config.configs['helpers.auto_fields.AutoLang'].auto_lang = "en"
        self.image_server_path = image_server_path
        self.manifest_bucket = manifest_bucket
        self.video_location = video_location
        self.manifest_data = manifest_data
        self.metadata = self.__build_metadata(manifest_data['metadata'])
        self.features = self.__find_features()
        self.manifest = self.__build_manifest()
        self.extensions = load_bundled_extensions(
            extensions=extensions
        )

    def __build_manifest(self):
        # @Todo: Clean up this method.  It's a mess and doing too much.
        homepage = HomePage(
            self.manifest_data['manifest_title'] if self.manifest_data['manifest_title'] != "" else "Untitled"
        ).body
        rendering = StudyGuide(self.manifest_data['metadata']['Artist']).body
        rights = "http://rightsstatements.org/vocab/InC/1.0/"
        if self.features:
            navplace_data = NavPlace(
                self.features,
                self.manifest_bucket,
                self.manifest_data['manifest_title'] if self.manifest_data['manifest_title'] != "" else "Untitled"
            ).features
            manifest = Manifest(
                id=f"{self.manifest_bucket}{self.manifest_data['id']}.json",
                label=self.manifest_data['manifest_title'] if self.manifest_data['manifest_title'] != "" else "Untitled",
                metadata=self.metadata,
                navPlace={"features": navplace_data},
                homepage=[homepage],
                rights=rights,
                rendering=rendering
            )
        else:
            manifest = Manifest(
                id=f"{self.manifest_bucket}{self.manifest_data['id']}.json",
                label=self.manifest_data['manifest_title'] if self.manifest_data[
                                                                  'manifest_title'] != "" else "Untitled",
                metadata=self.metadata,
                homepage=[homepage],
                rights=rights,
                rendering=rendering
            )
        for canvas in self.manifest_data['canvases']:
            thumbnail = Thumbnail(f"{self.image_server_path}{canvas['thumbnail']}").get()
            if canvas['type'] == 'Image':
                try:
                    # @Todo: Protect anno page and annotation
                    manifest.make_canvas_from_iiif(
                        url=f"{self.image_server_path}{canvas['key']}",
                        label=canvas['label'] if canvas['label'] != "" else "Untitled",
                        id=f"{self.manifest_bucket}{canvas['key']}/canvas/{canvas['sequence']}",
                        anno_id=f"{self.manifest_bucket}{canvas['key']}/canvas/{canvas['sequence']}/annotation/1",
                        anno_page_id=f"{self.manifest_bucket}{canvas['key']}/canvas/{canvas['sequence']}/annotation/1/page/1",
                        thumbnail=thumbnail,
                        metadata=self.__build_metadata(canvas.get('metadata'))
                    )
                except requests.HTTPError as e:
                    print(f'{e}. Missing file in bucket or other image server problem.')
                    error_message = str(e)
                    with open('errors.log', 'a') as f:
                        f.write(f'{error_message.split(" ")[-1]}\n')
            elif canvas['type'] == 'Video':
                vid_canvas = manifest.make_canvas(
                    id=f"{self.manifest_bucket}{canvas['key']}/canvas/{canvas['sequence']}",
                    label=canvas['label'] if canvas['label'] != "" else "Untitled",
                    thumbnail=thumbnail,
                    metadata=self.__build_metadata(canvas.get('metadata'))
                )
                details = self.__create_video_canvas(
                    canvas=vid_canvas,
                    canvas_data= canvas
                )
                vid_canvas.set_hwd(**details[1])
                vid_canvas.add_item(details[0])
                anno = vid_canvas.make_annotation(
                    id=f"{self.manifest_bucket}{canvas['key']}/canvas/caption/{canvas['sequence']}",
                    motivation="supplementing",
                    body={
                        "id": f"{self.manifest_bucket.replace("manifests/manifests/", "manifests/captions/")}{self.manifest_data['id']}_oo_base.vtt",
                        "type": "Text",
                        "language": "en",
                        "format": "text/vtt",
                        "label": {
                            "en": [
                                "Captions in English"
                            ]
                        }
                    },
                    target=f"{self.manifest_bucket}{canvas['key']}/canvas/caption/{canvas['sequence']}",
                    anno_page_id=f"{self.manifest_bucket}{canvas['key']}/canvas/{canvas['sequence']}/caption/annotation/1/page/1"
                )
        x = manifest.json(indent=2)
        manifest_as_json = json.loads(x)
        manifest_as_json['@context'] = ["http://iiif.io/api/extension/navplace/context.json", "http://iiif.io/api/presentation/3/context.json"]
        return manifest_as_json

    def __find_features(self):
        all_features = []
        for k, v in self.manifest_data['metadata'].items():
            if k == 'Geography':
                for feature in v:
                    all_features.append(feature)
        return all_features

    @staticmethod
    def __build_metadata(metadata_values):
        metadata = []
        for k, v in metadata_values.items():
            metadata.append(
                KeyValueString(
                    label=k,
                    value={"en": v}
                )
            )
        if not metadata:
            metadata.append(
                KeyValueString(
                    label="Missing metadata?",
                    value={"en": ["Yes"]}
                )
            )
        return metadata

    def write(self, path):
        with open(f'{path}/{self.manifest_data["id"]}.json', 'w') as outfile:
            outfile.write(
                json.dumps(
                    self.manifest, indent=2)
            )

    def __create_video_canvas(self, canvas, canvas_data):
        anno_body = ResourceItem(
            id=f"{self.video_location}{canvas_data['key']}",
            type="Video",
            format="video/mp4"
        )
        anno_page = AnnotationPage(
            id=f"{self.manifest_bucket}{canvas_data['key']}/canvas/{canvas_data['sequence']}/annotation/1/page/1"
        )
        anno = Annotation(
            id=f"{self.manifest_bucket}{canvas_data['key']}/canvas/{canvas_data['sequence']}/annotation/1",
            motivation="painting",
            body=anno_body,
            target=canvas.id
        )
        hwd = {"height": 1080, "width": 1920, "duration": canvas_data['duration']}
        anno_body.set_hwd(**hwd)
        anno_page.add_item(anno)
        return anno_page, hwd



import requests
from iiif_prezi3 import ResourceItem

class Thumbnail:
    def __init__(self, image_path):
        self.image_path = image_path
        self.best_size = self.__get_best_size()
        self.full_path = f"{self.image_path}/full/{self.best_size.get('width')},{self.best_size.get('height')}/0/default.jpg"

    def __get_best_size(self):
        try:
            r = requests.get(f"{self.image_path}/info.json").json()
            return r['sizes'][-3]
        except requests.exceptions.JSONDecodeError:
            return {'width': 100, 'height': 100}
        except IndexError:
            return r['sizes'][-1]

    def get(self):
        resource = ResourceItem(
            id=self.full_path,
            type="Image",
            format="image/jpeg",
            width=int(self.best_size.get('width')),
            height=int(self.best_size.get('height'))
        )
        resource.make_service(
            id=self.image_path,
            type="ImageService3",
            profile="level2"
        )
        return [resource]

    def __str__(self):
        return str(self.get())
